@@ -5,22 +5,26 @@
 #include "abstractaccount.h"
 
 #include "accountmanager.h"
+#include "network/networkcontroller.h"
 #include "preferences.h"
 #include "relationship.h"
-#include "utils/messagefiltercontainer.h"
 #include "tokodon_debug.h"
+#include "utils/messagefiltercontainer.h"
 #include <KLocalizedString>
 #include <QFile>
 #include <QHttpMultiPart>
 #include <QNetworkReply>
 #include <QUrlQuery>
 
+using namespace Qt::Literals::StringLiterals;
+
 AbstractAccount::AbstractAccount(QObject *parent, const QString &instanceUri)
     : QObject(parent)
     , m_instance_uri(instanceUri)
     // default to 500, instances which support more signal it
     , m_maxPostLength(500)
     , m_maxPollOptions(4)
+    , m_supportsLocalVisibility(false)
     , m_charactersReservedPerUrl(23)
     , m_identity(std::make_shared<Identity>())
     , m_allowedContentTypes(AllowedContentType::PlainText)
@@ -32,12 +36,22 @@ AbstractAccount::AbstractAccount(QObject *parent)
     // default to 500, instances which support more signal it
     , m_maxPostLength(500)
     , m_maxPollOptions(4)
+    , m_supportsLocalVisibility(false)
     , m_charactersReservedPerUrl(23)
     , m_identity(std::make_shared<Identity>())
     , m_allowedContentTypes(AllowedContentType::PlainText)
 {
 }
 
+AccountConfig *AbstractAccount::config()
+{
+    if (!m_config) {
+        m_config = new AccountConfig{settingsGroupName(), this};
+    }
+    Q_ASSERT(m_config != nullptr);
+    return m_config;
+}
+
 Preferences *AbstractAccount::preferences() const
 {
     return m_preferences;
@@ -67,6 +81,11 @@ size_t AbstractAccount::maxPollOptions() const
     return m_maxPollOptions;
 }
 
+bool AbstractAccount::supportsLocalVisibility() const
+{
+    return m_supportsLocalVisibility;
+}
+
 size_t AbstractAccount::charactersReservedPerUrl() const
 {
     return m_charactersReservedPerUrl;
@@ -95,27 +114,30 @@ bool AbstractAccount::hasInstanceUrl() const
 QUrl AbstractAccount::apiUrl(const QString &path) const
 {
     QUrl url = QUrl::fromUserInput(m_instance_uri);
-    url.setScheme("https");
+    url.setScheme(QStringLiteral("https"));
     url.setPath(path);
 
     return url;
 }
 
-void AbstractAccount::registerApplication(const QString &appName, const QString &website)
+void AbstractAccount::registerApplication(const QString &appName, const QString &website, const QString &additionalScopes)
 {
     // clear any previous bearer token credentials
     m_token = QString();
 
     // register
-    const QUrl regUrl = apiUrl("/api/v1/apps");
-    const QJsonObject obj = {
-        {"client_name", appName},
-        {"redirect_uris", "urn:ietf:wg:oauth:2.0:oob"},
-        {"scopes", "read write follow admin:read admin:write"},
-        {"website", website},
+    const QUrl regUrl = apiUrl(QStringLiteral("/api/v1/apps"));
+    const QJsonObject obj{
+        {QStringLiteral("client_name"), appName},
+        {QStringLiteral("redirect_uris"), QStringLiteral("tokodon://oauth")},
+        {QStringLiteral("scopes"), QStringLiteral("read write follow push %1").arg(additionalScopes)},
+        {QStringLiteral("website"), website},
     };
     const QJsonDocument doc(obj);
 
+    // Store for future usage (e.g. authorizeUrl)
+    m_additionalScopes = additionalScopes;
+
     post(regUrl, doc, false, this, [=](QNetworkReply *reply) {
         if (!reply->isFinished()) {
             qCDebug(TOKODON_LOG) << "not finished";
@@ -125,17 +147,70 @@ void AbstractAccount::registerApplication(const QString &appName, const QString
         const auto data = reply->readAll();
         const auto doc = QJsonDocument::fromJson(data);
 
-        m_client_id = doc.object()["client_id"].toString();
-        m_client_secret = doc.object()["client_secret"].toString();
+        m_client_id = doc.object()["client_id"_L1].toString();
+        m_client_secret = doc.object()["client_secret"_L1].toString();
 
-        s_messageFilter->insert(m_client_secret, "CLIENT_SECRET");
+        s_messageFilter->insert(m_client_secret, QStringLiteral("CLIENT_SECRET"));
 
         if (isRegistered()) {
             Q_EMIT registered();
+            fetchInstanceMetadata();
         }
     });
 }
 
+void AbstractAccount::registerAccount(const QString &username,
+                                      const QString &email,
+                                      const QString &password,
+                                      bool agreement,
+                                      const QString &locale,
+                                      const QString &reason)
+{
+    // get an app-level access token, obviously we don't have a user token yet.
+    const QUrl tokenUrl = getTokenUrl();
+    QUrlQuery q = buildOAuthQuery();
+
+    q.addQueryItem(QStringLiteral("client_secret"), m_client_secret);
+    q.addQueryItem(QStringLiteral("grant_type"), QStringLiteral("client_credentials"));
+    q.addQueryItem(QStringLiteral("scope"), QStringLiteral("write"));
+
+    post(tokenUrl, q, false, this, [=](QNetworkReply *reply) {
+        auto data = reply->readAll();
+        auto doc = QJsonDocument::fromJson(data);
+
+        // override the token for now
+        m_token = doc.object()["access_token"_L1].toString();
+        s_messageFilter->insert(m_token, QStringLiteral("ACCESS_TOKEN"));
+
+        const QUrlQuery formdata{
+            {QStringLiteral("username"), username},
+            {QStringLiteral("email"), email},
+            {QStringLiteral("password"), password},
+            {QStringLiteral("agreement"), agreement ? QStringLiteral("1") : QStringLiteral("0")},
+            {QStringLiteral("locale"), locale},
+            {QStringLiteral("reason"), reason},
+        };
+
+        post(
+            apiUrl(QStringLiteral("/api/v1/accounts")),
+            formdata,
+            true,
+            this,
+            [=](QNetworkReply *reply) {
+                const auto data = reply->readAll();
+                const auto doc = QJsonDocument::fromJson(data);
+
+                if (doc.object().contains("access_token"_L1)) {
+                    setUsername(username);
+                    setAccessToken(doc["access_token"_L1].toString());
+                }
+            },
+            [=](QNetworkReply *reply) {
+                Q_EMIT registrationError(QString::fromUtf8(reply->readAll()));
+            });
+    });
+}
+
 Identity *AbstractAccount::identity()
 {
     return m_identity.get();
@@ -188,36 +263,45 @@ bool AbstractAccount::identityCached(const QString &accountId) const
 
 QUrlQuery AbstractAccount::buildOAuthQuery() const
 {
-    return QUrlQuery{{"client_id", m_client_id}};
+    return QUrlQuery{{QStringLiteral("client_id"), m_client_id}};
 }
 
 QUrl AbstractAccount::getAuthorizeUrl() const
 {
-    QUrl url = apiUrl("/oauth/authorize");
+    QUrl url = apiUrl(QStringLiteral("/oauth/authorize"));
     QUrlQuery q = buildOAuthQuery();
 
-    q.addQueryItem("redirect_uri", "urn:ietf:wg:oauth:2.0:oob");
-    q.addQueryItem("response_type", "code");
-    q.addQueryItem("scope", "read write follow admin:read admin:write");
+    q.addQueryItem(QStringLiteral("redirect_uri"), QStringLiteral("tokodon://oauth"));
+    q.addQueryItem(QStringLiteral("response_type"), QStringLiteral("code"));
+    q.addQueryItem(QStringLiteral("scope"), QStringLiteral("read write follow push ") + m_additionalScopes);
 
     url.setQuery(q);
 
     return url;
 }
 
+void AbstractAccount::setAccessToken(const QString &token)
+{
+    m_token = token;
+    s_messageFilter->insert(m_token, QStringLiteral("ACCESS_TOKEN"));
+    AccountManager::instance().addAccount(this, false);
+    AccountManager::instance().selectAccount(this, true);
+    validateToken();
+    writeToSettings();
+}
+
 QUrl AbstractAccount::getTokenUrl() const
 {
-    return apiUrl("/oauth/token");
+    return apiUrl(QStringLiteral("/oauth/token"));
 }
 
 void AbstractAccount::setInstanceUri(const QString &instance_uri)
 {
     // instance URI changed, get new credentials
     QUrl instance_url = QUrl::fromUserInput(instance_uri);
-    instance_url.setScheme("https"); // getting token from http is not supported
+    instance_url.setScheme(QStringLiteral("https")); // getting token from http is not supported
 
     m_instance_uri = instance_url.toString();
-    registerApplication("Tokodon", "https://apps.kde.org/tokodon");
 }
 
 QString AbstractAccount::instanceUri() const
@@ -230,26 +314,22 @@ void AbstractAccount::setToken(const QString &authcode)
     const QUrl tokenUrl = getTokenUrl();
     QUrlQuery q = buildOAuthQuery();
 
-    q.addQueryItem("client_secret", m_client_secret);
-    q.addQueryItem("redirect_uri", "urn:ietf:wg:oauth:2.0:oob");
-    q.addQueryItem("grant_type", "authorization_code");
-    q.addQueryItem("code", authcode);
+    q.addQueryItem(QStringLiteral("client_secret"), m_client_secret);
+    q.addQueryItem(QStringLiteral("redirect_uri"), QStringLiteral("tokodon://oauth"));
+    q.addQueryItem(QStringLiteral("grant_type"), QStringLiteral("authorization_code"));
+    q.addQueryItem(QStringLiteral("code"), authcode);
 
     post(tokenUrl, q, false, this, [=](QNetworkReply *reply) {
         auto data = reply->readAll();
         auto doc = QJsonDocument::fromJson(data);
 
-        m_token = doc.object()["access_token"].toString();
-        s_messageFilter->insert(m_token, "ACCESS_TOKEN");
-        AccountManager::instance().addAccount(this);
-        AccountManager::instance().selectAccount(this, true);
-        validateToken();
+        setAccessToken(doc.object()["access_token"_L1].toString());
     });
 }
 
-void AbstractAccount::mutatePost(Post *p, const QString &verb, bool deliver_home)
+void AbstractAccount::mutatePost(const QString &id, const QString &verb, bool deliver_home)
 {
-    const QUrl mutation_url = apiUrl(QString("/api/v1/statuses/%1/%2").arg(p->postId(), verb));
+    const QUrl mutation_url = apiUrl(QStringLiteral("/api/v1/statuses/%1/%2").arg(id, verb));
     const QJsonDocument doc;
 
     post(mutation_url, doc, true, this, [=](QNetworkReply *reply) {
@@ -263,49 +343,49 @@ void AbstractAccount::mutatePost(Post *p, const QString &verb, bool deliver_home
             auto p = new Post(this, obj, this);
             posts.push_back(p);
 
-            Q_EMIT fetchedTimeline("home", posts);
+            Q_EMIT fetchedTimeline(QStringLiteral("home"), posts);
         }
     });
 }
 
 void AbstractAccount::favorite(Post *p)
 {
-    mutatePost(p, "favourite");
+    mutatePost(p->postId(), QStringLiteral("favourite"));
 }
 
 void AbstractAccount::unfavorite(Post *p)
 {
-    mutatePost(p, "unfavourite");
+    mutatePost(p->postId(), QStringLiteral("unfavourite"));
 }
 
 void AbstractAccount::repeat(Post *p)
 {
-    mutatePost(p, "reblog", true);
+    mutatePost(p->postId(), QStringLiteral("reblog"), true);
 }
 
 void AbstractAccount::unrepeat(Post *p)
 {
-    mutatePost(p, "unreblog");
+    mutatePost(p->postId(), QStringLiteral("unreblog"));
 }
 
 void AbstractAccount::bookmark(Post *p)
 {
-    mutatePost(p, "bookmark", true);
+    mutatePost(p->postId(), QStringLiteral("bookmark"), true);
 }
 
 void AbstractAccount::unbookmark(Post *p)
 {
-    mutatePost(p, "unbookmark");
+    mutatePost(p->postId(), QStringLiteral("unbookmark"));
 }
 
 void AbstractAccount::pin(Post *p)
 {
-    mutatePost(p, "pin", true);
+    mutatePost(p->postId(), QStringLiteral("pin"), true);
 }
 
 void AbstractAccount::unpin(Post *p)
 {
-    mutatePost(p, "unpin");
+    mutatePost(p->postId(), QStringLiteral("unpin"));
 }
 
 // It seemed clearer to keep this logic separate from the general instance metadata collection, on the off chance
@@ -315,7 +395,7 @@ static AbstractAccount::AllowedContentType parseVersion(const QString &instanceV
     using ContentType = AbstractAccount::AllowedContentType;
 
     unsigned int result = ContentType::PlainText;
-    if (instanceVer.contains("glitch")) {
+    if (instanceVer.contains("glitch"_L1)) {
         result |= ContentType::Markdown | ContentType::Html;
     }
 
@@ -323,10 +403,10 @@ static AbstractAccount::AllowedContentType parseVersion(const QString &instanceV
 }
 
 static QMap<QString, AbstractAccount::AllowedContentType> stringToContentType = {
-    {"text/plain", AbstractAccount::AllowedContentType::PlainText},
-    {"text/bbcode", AbstractAccount::AllowedContentType::BBCode},
-    {"text/html", AbstractAccount::AllowedContentType::Html},
-    {"text/markdown", AbstractAccount::AllowedContentType::Markdown},
+    {QStringLiteral("text/plain"), AbstractAccount::AllowedContentType::PlainText},
+    {QStringLiteral("text/bbcode"), AbstractAccount::AllowedContentType::BBCode},
+    {QStringLiteral("text/html"), AbstractAccount::AllowedContentType::Html},
+    {QStringLiteral("text/markdown"), AbstractAccount::AllowedContentType::Markdown},
 };
 
 static AbstractAccount::AllowedContentType parsePleromaInfo(const QJsonDocument &doc)
@@ -335,12 +415,12 @@ static AbstractAccount::AllowedContentType parsePleromaInfo(const QJsonDocument
     unsigned int result = ContentType::PlainText;
 
     auto obj = doc.object();
-    if (obj.contains("metadata")) {
-        auto metadata = obj["metadata"].toObject();
-        if (!metadata.contains("postFormats"))
+    if (obj.contains("metadata"_L1)) {
+        auto metadata = obj["metadata"_L1].toObject();
+        if (!metadata.contains("postFormats"_L1))
             return static_cast<ContentType>(result);
 
-        auto formats = metadata["postFormats"].toArray();
+        auto formats = metadata["postFormats"_L1].toArray();
 
         for (auto c : formats) {
             auto fmt = c.toString();
@@ -357,59 +437,127 @@ static AbstractAccount::AllowedContentType parsePleromaInfo(const QJsonDocument
 
 void AbstractAccount::fetchInstanceMetadata()
 {
-    QUrl instance_url = apiUrl("/api/v1/instance");
-    QUrl pleroma_info = apiUrl("/nodeinfo/2.1.json");
+    get(
+        apiUrl(QStringLiteral("/api/v2/instance")),
+        false,
+        this,
+        [=](QNetworkReply *reply) {
+            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+                return;
 
-    get(instance_url, false, this, [=](QNetworkReply *reply) {
-        if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
-            return;
+            const auto data = reply->readAll();
+            const auto doc = QJsonDocument::fromJson(data);
 
-        const auto data = reply->readAll();
-        const auto doc = QJsonDocument::fromJson(data);
+            if (!doc.isObject())
+                return;
 
-        if (!doc.isObject())
-            return;
+            const auto obj = doc.object();
 
-        const auto obj = doc.object();
+            if (obj.contains("configuration"_L1)) {
+                const auto configObj = obj["configuration"_L1].toObject();
 
-        if (obj.contains("configuration")) {
-            const auto configObj = obj["configuration"].toObject();
-
-            if (configObj.contains("statuses")) {
-                const auto statusConfigObj = configObj["statuses"].toObject();
-                m_maxPostLength = statusConfigObj["max_characters"].toInt();
-                m_charactersReservedPerUrl = statusConfigObj["characters_reserved_per_url"].toInt();
+                if (configObj.contains("statuses"_L1)) {
+                    const auto statusConfigObj = configObj["statuses"_L1].toObject();
+                    m_maxPostLength = statusConfigObj["max_characters"_L1].toInt();
+                    m_charactersReservedPerUrl = statusConfigObj["characters_reserved_per_url"_L1].toInt();
+                }
             }
-        }
 
-        // One can only hope that there will always be a version attached
-        if (obj.contains("version")) {
-            m_allowedContentTypes = parseVersion(obj["version"].toString());
-        }
+            // One can only hope that there will always be a version attached
+            if (obj.contains("version"_L1)) {
+                m_allowedContentTypes = parseVersion(obj["version"_L1].toString());
+            }
 
-        // Pleroma/Akkoma may report maximum post characters here, instead
-        if (obj.contains("max_toot_chars")) {
-            m_maxPostLength = obj["max_toot_chars"].toInt();
-        }
+            // Pleroma/Akkoma may report maximum post characters here, instead
+            if (obj.contains("max_toot_chars"_L1)) {
+                m_maxPostLength = obj["max_toot_chars"_L1].toInt();
+            }
 
-        // Pleroma/Akkoma can report higher poll limits
-        if (obj.contains("poll_limits")) {
-            m_maxPollOptions = obj["poll_limits"].toObject()["max_options"].toInt();
-        }
+            // Pleroma/Akkoma can report higher poll limits
+            if (obj.contains("poll_limits"_L1)) {
+                m_maxPollOptions = obj["poll_limits"_L1].toObject()["max_options"_L1].toInt();
+            }
 
-        m_instance_name = obj["title"].toString();
-        Q_EMIT fetchedInstanceMetadata();
-    });
-    m_instance_name = QString("social");
-    Q_EMIT fetchedInstanceMetadata();
+            // Other instance of poll options
+            if (obj.contains("polls"_L1)) {
+                m_maxPollOptions = obj["polls"_L1].toObject()["max_options"_L1].toInt();
+            }
 
-    get(pleroma_info, false, this, [=](QNetworkReply *reply) {
+            if (obj.contains("registrations"_L1)) {
+                m_registrationsOpen = obj["registrations"_L1].toObject()["enabled"_L1].toBool();
+                m_registrationMessage = obj["registrations"_L1].toObject()["message"_L1].toString();
+            }
+
+            m_supportsLocalVisibility = obj.contains("pleroma"_L1);
+
+            m_instance_name = obj["title"_L1].toString();
+
+            Q_EMIT fetchedInstanceMetadata();
+        },
+        [=](QNetworkReply *) {
+            // Fall back to v1 instance information
+            // TODO: a lot of this can be merged with v2 handling
+            get(apiUrl(QStringLiteral("/api/v1/instance")), false, this, [=](QNetworkReply *reply) {
+                if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+                    return;
+
+                const auto data = reply->readAll();
+                const auto doc = QJsonDocument::fromJson(data);
+
+                if (!doc.isObject())
+                    return;
+
+                const auto obj = doc.object();
+
+                if (obj.contains("configuration"_L1)) {
+                    const auto configObj = obj["configuration"_L1].toObject();
+
+                    if (configObj.contains("statuses"_L1)) {
+                        const auto statusConfigObj = configObj["statuses"_L1].toObject();
+                        m_maxPostLength = statusConfigObj["max_characters"_L1].toInt();
+                        m_charactersReservedPerUrl = statusConfigObj["characters_reserved_per_url"_L1].toInt();
+                    }
+                }
+
+                // One can only hope that there will always be a version attached
+                if (obj.contains("version"_L1)) {
+                    m_allowedContentTypes = parseVersion(obj["version"_L1].toString());
+                }
+
+                // Pleroma/Akkoma may report maximum post characters here, instead
+                if (obj.contains("max_toot_chars"_L1)) {
+                    m_maxPostLength = obj["max_toot_chars"_L1].toInt();
+                }
+
+                // Pleroma/Akkoma can report higher poll limits
+                if (obj.contains("poll_limits"_L1)) {
+                    m_maxPollOptions = obj["poll_limits"_L1].toObject()["max_options"_L1].toInt();
+                }
+
+                // Other instance of poll options
+                if (obj.contains("polls"_L1)) {
+                    m_maxPollOptions = obj["polls"_L1].toObject()["max_options"_L1].toInt();
+                }
+
+                m_registrationsOpen = obj["registrations"_L1].toBool();
+
+                m_supportsLocalVisibility = obj.contains("pleroma"_L1);
+
+                m_instance_name = obj["title"_L1].toString();
+
+                Q_EMIT fetchedInstanceMetadata();
+            });
+        });
+
+    get(apiUrl(QStringLiteral("/nodeinfo/2.1.json")), false, this, [=](QNetworkReply *reply) {
         const auto data = reply->readAll();
         const auto doc = QJsonDocument::fromJson(data);
 
         m_allowedContentTypes = parsePleromaInfo(doc);
         Q_EMIT fetchedInstanceMetadata();
     });
+
+    fetchCustomEmojis();
 }
 
 void AbstractAccount::invalidate()
@@ -424,12 +572,12 @@ void AbstractAccount::invalidatePost(Post *p)
 
 QUrl AbstractAccount::streamingUrl(const QString &stream)
 {
-    QUrl url = apiUrl("/api/v1/streaming");
+    QUrl url = apiUrl(QStringLiteral("/api/v1/streaming"));
     url.setQuery(QUrlQuery{
-        {"access_token", m_token},
-        {"stream", stream},
+        {QStringLiteral("access_token"), m_token},
+        {QStringLiteral("stream"), stream},
     });
-    url.setScheme("wss");
+    url.setScheme(QStringLiteral("wss"));
 
     return url;
 }
@@ -439,6 +587,11 @@ void AbstractAccount::handleNotification(const QJsonDocument &doc)
     const auto obj = doc.object();
     std::shared_ptr<Notification> n = std::make_shared<Notification>(this, obj);
 
+    if (n->type() == Notification::FollowRequest) {
+        m_hasFollowRequests = true;
+        Q_EMIT hasFollowRequestsChanged();
+    }
+
     Q_EMIT notification(n);
 }
 
@@ -467,7 +620,7 @@ void AbstractAccount::executeAction(Identity *identity, AccountAction accountAct
         auto jsonObj = doc.object();
 
         // Check if the request failed due to one account blocking the other
-        if (!jsonObj.value("error").isUndefined()) {
+        if (!jsonObj.value("error"_L1).isUndefined()) {
             const QHash<AccountAction, QString> accountActionMap = {
                 {AccountAction::Follow, i18n("Could not follow account")},
                 {AccountAction::Unfollow, i18n("Could not unfollow account")},
@@ -498,8 +651,8 @@ void AbstractAccount::followAccount(Identity *identity, bool reblogs, bool notif
     executeAction(identity,
                   AccountAction::Follow,
                   {
-                      {"reblogs", reblogs},
-                      {"notify", notify},
+                      {QStringLiteral("reblogs"), reblogs},
+                      {QStringLiteral("notify"), notify},
                   });
 }
 
@@ -520,7 +673,7 @@ void AbstractAccount::unblockAccount(Identity *identity)
 
 void AbstractAccount::muteAccount(Identity *identity, bool notifications, int duration)
 {
-    executeAction(identity, AccountAction::Mute, {{"notifcations", notifications}, {"duration", duration}});
+    executeAction(identity, AccountAction::Mute, {{QStringLiteral("notifcations"), notifications}, {QStringLiteral("duration"), duration}});
 }
 
 void AbstractAccount::unmuteAccount(Identity *identity)
@@ -543,15 +696,42 @@ void AbstractAccount::addNote(Identity *identity, const QString &note)
     if (note.isEmpty()) {
         executeAction(identity, AccountAction::Note);
     } else {
-        executeAction(identity, AccountAction::Note, {{"comment", note}});
+        executeAction(identity, AccountAction::Note, {{QStringLiteral("comment"), note}});
     }
 }
 
+void AbstractAccount::mutateRemotePost(const QString &url, const QString &verb)
+{
+    NetworkController::instance().requestRemoteObject(this, url, [=](QNetworkReply *reply) {
+        const auto searchResult = QJsonDocument::fromJson(reply->readAll()).object();
+        const auto statuses = searchResult[QStringLiteral("statuses")].toArray();
+        const auto accounts = searchResult[QStringLiteral("accounts")].toArray();
+
+        // TODO: emit error when the mutation has failed, no post is available on this account's server
+        if (!statuses.isEmpty()) {
+            const auto status = statuses[0].toObject();
+
+            const QString localID = status["id"_L1].toString();
+            mutatePost(localID, verb);
+        }
+    });
+}
+
 bool AbstractAccount::isRegistered() const
 {
     return !m_client_id.isEmpty() && !m_client_secret.isEmpty();
 }
 
+bool AbstractAccount::registrationsOpen() const
+{
+    return m_registrationsOpen;
+}
+
+QString AbstractAccount::registrationMessage() const
+{
+    return m_registrationMessage;
+}
+
 QString AbstractAccount::settingsGroupName() const
 {
     return AccountManager::settingsGroupName(m_name, m_instance_uri);
@@ -565,4 +745,41 @@ QString AbstractAccount::clientSecretKey() const
 QString AbstractAccount::accessTokenKey() const
 {
     return AccountManager::accessTokenKey(settingsGroupName());
-}
\ No newline at end of file
+}
+
+void AbstractAccount::fetchCustomEmojis()
+{
+    m_customEmojis.clear();
+
+    get(apiUrl(QStringLiteral("/api/v1/custom_emojis")), false, this, [=](QNetworkReply *reply) {
+        if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+            return;
+
+        const auto data = reply->readAll();
+        const auto doc = QJsonDocument::fromJson(data);
+
+        if (!doc.isArray())
+            return;
+
+        const auto array = doc.array();
+
+        for (auto emojiObj : array) {
+            if (!emojiObj.isObject()) {
+                continue;
+            }
+
+            CustomEmoji customEmoji{};
+            customEmoji.shortcode = emojiObj[QStringLiteral("shortcode")].toString();
+            customEmoji.url = emojiObj[QStringLiteral("url")].toString();
+
+            m_customEmojis.push_back(customEmoji);
+        }
+
+        Q_EMIT fetchedCustomEmojis();
+    });
+}
+
+QList<CustomEmoji> AbstractAccount::customEmojis() const
+{
+    return m_customEmojis;
+}
