@@ -32,11 +32,9 @@ class Account : public AbstractAccount
     Q_OBJECT
 
 public:
-    explicit Account(const QString &instance_uri, QNetworkAccessManager *nam, bool ignoreSslErrors = false, QObject *parent = nullptr);
-    explicit Account(const AccountConfig &settings, QNetworkAccessManager *nam, QObject *parent = nullptr);
+    explicit Account(const QString &instance_uri, QNetworkAccessManager *nam, bool ignoreSslErrors = false, bool admin = true, QObject *parent = nullptr);
+    explicit Account(AccountConfig *settings, QNetworkAccessManager *nam, QObject *parent = nullptr);
     ~Account() override;
-
-    // making API calls
     void get(const QUrl &url,
              bool authenticated,
              QObject *parent,
@@ -49,9 +47,15 @@ public:
               std::function<void(QNetworkReply *)> callback,
               std::function<void(QNetworkReply *)> errorCallback,
               QHash<QByteArray, QByteArray> headers = {}) override;
-    void post(const QUrl &url, const QUrlQuery &formdata, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
+    void post(const QUrl &url,
+              const QUrlQuery &formdata,
+              bool authenticated,
+              QObject *parent,
+              std::function<void(QNetworkReply *)> callback,
+              std::function<void(QNetworkReply *)> errorCallback) override;
     QNetworkReply *post(const QUrl &url, QHttpMultiPart *message, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
     void put(const QUrl &url, const QJsonDocument &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
+    void put(const QUrl &url, const QUrlQuery &formdata, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
     void patch(const QUrl &url, QHttpMultiPart *multiPart, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)>) override;
     void deleteResource(const QUrl &url, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
     QNetworkReply *upload(const QUrl &filename, std::function<void(QNetworkReply *)> callback) override;
@@ -62,12 +66,9 @@ public:
         return m_qnam;
     }
 
-    // save/restore.
-    // writeToSettings assumes a settings object in a parent context
-    // buildFromSettings assumes a settings object in the object context
     void writeToSettings() override;
 
-    void buildFromSettings(const AccountConfig &settings) override;
+    void buildFromSettings() override;
 
     void validateToken() override;
 
@@ -75,11 +76,17 @@ public:
 
     Q_INVOKABLE void checkForFollowRequests() override;
 
+    Q_INVOKABLE void updatePushNotifications() override;
+
 private:
+    void unsubscribePushNotifications();
+    void subscribePushNotifications();
+    QUrlQuery buildNotificationFormData();
+
     bool m_ignoreSslErrors = false;
     QNetworkAccessManager *m_qnam;
     QMap<QString, QWebSocket *> m_websockets;
-    bool m_hasFollowRequests = false;
+    bool m_hasPushSubscription = false;
 
     // common parts for all HTTP request
     QNetworkRequest makeRequest(const QUrl &url, bool authenticated) const;
