@@ -8,25 +8,17 @@
 #include "config.h"
 #include "network/networkaccessmanagerfactory.h"
 #include "tokodon_debug.h"
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <qt5keychain/keychain.h>
-#else
 #include <qt6keychain/keychain.h>
-#endif
+
+#include <utility>
+
+using namespace Qt::Literals::StringLiterals;
 
 AccountManager::AccountManager(QObject *parent)
     : QAbstractListModel(parent)
     , m_selected_account(nullptr)
     , m_qnam(NetworkAccessManagerFactory().create(this))
 {
-    migrateSettings();
-    loadFromSettings();
-
-    connect(this, &AccountManager::accountSelected, this, [=](AbstractAccount *account) {
-        if (account != nullptr) {
-            account->checkForFollowRequests();
-        }
-    });
 }
 
 AccountManager::~AccountManager()
@@ -51,7 +43,10 @@ QVariant AccountManager::data(const QModelIndex &index, int role) const
     switch (role) {
     case Qt::DisplayRole:
     case DisplayNameRole:
-        return account->identity()->displayNameHtml();
+        if (!account->identity()->displayNameHtml().isEmpty())
+            return account->identity()->displayNameHtml();
+        else
+            return account->username();
     case DescriptionRole:
         return account->identity()->account();
     case InstanceRole:
@@ -85,9 +80,9 @@ AccountManager &AccountManager::instance()
     return accountManager;
 }
 
-AbstractAccount *AccountManager::createNewAccount(const QString &instanceUri, bool ignoreSslErrors)
+AbstractAccount *AccountManager::createNewAccount(const QString &instanceUri, bool ignoreSslErrors, bool admin)
 {
-    return new Account(instanceUri, m_qnam, ignoreSslErrors, this);
+    return new Account(instanceUri, m_qnam, ignoreSslErrors, admin, this);
 }
 
 bool AccountManager::hasAccounts() const
@@ -100,10 +95,19 @@ bool AccountManager::hasAnyAccounts() const
     return m_hasAnyAccounts;
 }
 
-void AccountManager::addAccount(AbstractAccount *account)
+void AccountManager::addAccount(AbstractAccount *account, bool skipAuthenticationCheck)
 {
     beginInsertRows(QModelIndex(), m_accounts.size(), m_accounts.size());
     m_accounts.append(account);
+    int acctIndex = m_accountStatus.size();
+    if (!skipAuthenticationCheck) {
+        if (m_testMode) {
+            m_accountStatus.push_back(AccountStatus::Loaded);
+        } else {
+            m_accountStatus.push_back(AccountStatus::NotLoaded);
+        }
+        m_accountStatusStrings.push_back({});
+    }
     endInsertRows();
 
     Q_EMIT accountAdded(account);
@@ -112,12 +116,20 @@ void AccountManager::addAccount(AbstractAccount *account)
         childIdentityChanged(account);
         account->writeToSettings();
     });
-    connect(account, &Account::authenticated, this, [this]() {
-        Q_EMIT dataChanged(index(0, 0), index(m_accounts.size() - 1, 0));
-    });
+    if (!skipAuthenticationCheck) {
+        connect(account, &Account::authenticated, this, [this, acctIndex](const bool authenticated, const QString &errorMessage) {
+            if (authenticated) {
+                m_accountStatus[acctIndex] = AccountStatus::Loaded;
+            } else {
+                m_accountStatus[acctIndex] = AccountStatus::InvalidCredentials;
+                m_accountStatusStrings[acctIndex] = errorMessage;
+            }
+            Q_EMIT dataChanged(index(0, 0), index(m_accounts.size() - 1, 0));
+        });
+    }
 
-    connect(account, &Account::fetchedTimeline, this, [this, account](QString original_name, QList<Post *> posts) {
-        Q_EMIT fetchedTimeline(account, original_name, posts);
+    connect(account, &Account::fetchedTimeline, this, [this, account](const QString &original_name, QList<Post *> posts) {
+        Q_EMIT fetchedTimeline(account, original_name, std::move(posts));
     });
     connect(account, &Account::invalidated, this, [this, account]() {
         Q_EMIT invalidated(account);
@@ -130,13 +142,17 @@ void AccountManager::addAccount(AbstractAccount *account)
         Q_EMIT invalidatedPost(account, p);
     });
     connect(account, &Account::notification, this, [this, account](std::shared_ptr<Notification> n) {
-        Q_EMIT notification(account, n);
+        Q_EMIT notification(account, std::move(n));
     });
 
     if (m_selected_account == nullptr) {
         m_selected_account = account;
         Q_EMIT accountSelected(m_selected_account);
     }
+
+    if (m_testMode) {
+        checkIfLoadingFinished();
+    }
 }
 
 void AccountManager::childIdentityChanged(AbstractAccount *account)
@@ -154,11 +170,11 @@ void AccountManager::removeAccount(AbstractAccount *account)
     config->deleteGroup(account->settingsGroupName());
     config->sync();
 
-    auto accessTokenJob = new QKeychain::DeletePasswordJob{"Tokodon"};
+    auto accessTokenJob = new QKeychain::DeletePasswordJob{QStringLiteral("Tokodon")};
     accessTokenJob->setKey(account->accessTokenKey());
     accessTokenJob->start();
 
-    auto clientSecretJob = new QKeychain::DeletePasswordJob{"Tokodon"};
+    auto clientSecretJob = new QKeychain::DeletePasswordJob{QStringLiteral("Tokodon")};
     clientSecretJob->setKey(account->clientSecretKey());
     clientSecretJob->start();
 
@@ -181,12 +197,42 @@ void AccountManager::removeAccount(AbstractAccount *account)
 void AccountManager::reloadAccounts()
 {
     for (auto account : std::as_const(m_accounts)) {
-        account->validateToken();
+        if (account->haveToken()) {
+            account->validateToken();
+        }
     }
 
     Q_EMIT accountsReloaded();
 }
 
+bool AccountManager::selectedAccountHasIssue() const
+{
+    if (!m_selected_account) {
+        return false;
+    }
+
+    const int index = m_accounts.indexOf(m_selected_account);
+    if (index != -1 && index < m_accountStatus.size()) {
+        return m_accountStatus[index] == AccountStatus::InvalidCredentials;
+    }
+
+    return false;
+}
+
+QString AccountManager::selectedAccountLoginIssue() const
+{
+    if (!m_selected_account) {
+        return {};
+    }
+
+    const int index = m_accounts.indexOf(m_selected_account);
+    if (index != -1) {
+        return m_accountStatusStrings[index];
+    }
+
+    return {};
+}
+
 void AccountManager::selectAccount(AbstractAccount *account, bool explicitUserAction)
 {
     if (!m_accounts.contains(account)) {
@@ -227,26 +273,31 @@ int AccountManager::selectedIndex() const
 
 void AccountManager::loadFromSettings()
 {
+    if (m_testMode) {
+        qCDebug(TOKODON_LOG) << "Test mode enabled, no local accounts are loaded.";
+        return;
+    }
+
     qCDebug(TOKODON_LOG) << "Loading accounts from settings.";
 
     auto config = KSharedConfig::openStateConfig();
     for (const auto &id : config->groupList()) {
-        if (id.contains('@')) {
-            auto accountConfig = AccountConfig{id};
+        if (id.contains('@'_L1)) {
+            auto accountConfig = new AccountConfig{id};
 
-            int index = m_accountStatus.size();
+            const int index = m_accountStatus.size();
             m_accountStatus.push_back(AccountStatus::NotLoaded);
+            m_accountStatusStrings.push_back({});
 
             auto account = new Account(accountConfig, m_qnam);
-            connect(account, &Account::authenticated, this, [=](bool successful) {
+            addAccount(account, true);
+
+            connect(account, &Account::authenticated, this, [this, account, index](const bool successful, const QString &errorMessage) {
                 if (successful && account->haveToken() && account->hasName() && account->hasInstanceUrl()) {
                     m_accountStatus[index] = AccountStatus::Loaded;
-
-                    addAccount(account);
                 } else {
                     m_accountStatus[index] = AccountStatus::InvalidCredentials;
-
-                    delete account;
+                    m_accountStatusStrings[index] = errorMessage;
                 }
 
                 checkIfLoadingFinished();
@@ -294,7 +345,7 @@ void AccountManager::checkIfLoadingFinished()
     for (auto account : m_accounts) {
         // old LastUsedAccount values used to be only username
         const bool isOldVersion = !config->lastUsedAccount().contains(QLatin1Char('@'));
-        const bool isEmpty = config->lastUsedAccount().isEmpty() || config->lastUsedAccount() == '@';
+        const bool isEmpty = config->lastUsedAccount().isEmpty() || config->lastUsedAccount() == '@'_L1;
         const bool matchesNewFormat = account->settingsGroupName() == config->lastUsedAccount();
         const bool matchesOldFormat = account->username() == config->lastUsedAccount();
 
@@ -340,6 +391,10 @@ QString AccountManager::accessTokenKey(const QString &name)
 
 void AccountManager::migrateSettings()
 {
+    if (m_testMode) {
+        return;
+    }
+
     QSettings settings;
 
     const auto version = settings.value("settingsVersion", -1).toInt();
@@ -397,12 +452,12 @@ void AccountManager::migrateSettings()
 
             config.save();
 
-            auto accessTokenJob = new QKeychain::WritePasswordJob{"Tokodon"};
+            auto accessTokenJob = new QKeychain::WritePasswordJob{QStringLiteral("Tokodon")};
             accessTokenJob->setKey(AccountManager::accessTokenKey(settingsGroupName));
             accessTokenJob->setTextData(settings.value("token").toString());
             accessTokenJob->start();
 
-            auto clientSecretJob = new QKeychain::WritePasswordJob{"Tokodon"};
+            auto clientSecretJob = new QKeychain::WritePasswordJob{QStringLiteral("Tokodon")};
             clientSecretJob->setKey(AccountManager::clientSecretKey(settingsGroupName));
             clientSecretJob->setTextData(settings.value("client_secret").toString());
             clientSecretJob->start();
@@ -416,3 +471,71 @@ void AccountManager::migrateSettings()
         settings.clear();
     }
 }
+
+bool AccountManager::isFlatpak() const
+{
+#ifdef TOKODON_FLATPAK
+    return true;
+#else
+    return false;
+#endif
+}
+
+void AccountManager::setTestMode(const bool enabled)
+{
+    m_testMode = enabled;
+}
+
+bool AccountManager::testMode() const
+{
+    return m_testMode;
+}
+
+void AccountManager::queueNotifications()
+{
+    static int accountsLeft = m_accounts.size();
+
+    const auto checkIfDone = [this]() {
+        if (accountsLeft <= 0) {
+            Q_EMIT finishedNotificationQueue();
+        }
+    };
+
+    for (auto account : m_accounts) {
+        QUrl uri;
+        uri = QUrl::fromUserInput(account->instanceUri());
+        uri.setPath(QStringLiteral("/api/v1/notifications"));
+
+        AccountConfig config(account->settingsGroupName());
+
+        QUrlQuery urlQuery(uri);
+        urlQuery.addQueryItem(QStringLiteral("limit"), QString::number(1));
+        if (!config.lastPushNotification().isEmpty()) {
+            urlQuery.addQueryItem(QStringLiteral("min_id"), config.lastPushNotification());
+        }
+        uri.setQuery(urlQuery);
+
+        account->get(uri, true, this, [this, account, checkIfDone](QNetworkReply *reply) {
+            const auto data = reply->readAll();
+            const auto doc = QJsonDocument::fromJson(data);
+
+            if (!doc.isArray() || doc.array().isEmpty()) {
+                accountsLeft--;
+                checkIfDone();
+                return;
+            }
+
+            const auto obj = doc.array()[0].toObject();
+            std::shared_ptr<Notification> n = std::make_shared<Notification>(account, obj);
+
+            AccountConfig config(account->settingsGroupName());
+            config.setLastPushNotification(obj["id"_L1].toString());
+            config.save();
+
+            Q_EMIT account->notification(n);
+
+            accountsLeft--;
+            checkIfDone();
+        });
+    }
+}
