@@ -9,6 +9,8 @@
 #include <QHttpMultiPart>
 #include <QMimeDatabase>
 
+using namespace Qt::Literals::StringLiterals;
+
 ProfileEditorBackend::ProfileEditorBackend(QObject *parent)
     : QObject(parent)
 {
@@ -38,6 +40,11 @@ QString ProfileEditorBackend::displayName() const
     return m_displayName;
 }
 
+QString ProfileEditorBackend::displayNameHtml() const
+{
+    return CustomEmoji::replaceCustomEmojis(m_account->customEmojis(), m_displayName);
+}
+
 void ProfileEditorBackend::setDisplayName(const QString &displayName)
 {
     if (m_displayName == displayName) {
@@ -61,34 +68,6 @@ void ProfileEditorBackend::setNote(const QString &note)
     Q_EMIT noteChanged();
 }
 
-bool ProfileEditorBackend::sensitive() const
-{
-    return m_sensitive;
-}
-
-void ProfileEditorBackend::setSensitive(bool sensitive)
-{
-    if (m_sensitive == sensitive) {
-        return;
-    }
-    m_sensitive = sensitive;
-    Q_EMIT sensitiveChanged();
-}
-
-QString ProfileEditorBackend::privacy() const
-{
-    return m_privacy;
-}
-
-void ProfileEditorBackend::setPrivacy(const QString &privacy)
-{
-    if (privacy == m_privacy) {
-        return;
-    }
-    m_privacy = privacy;
-    Q_EMIT privacyChanged();
-}
-
 bool ProfileEditorBackend::bot() const
 {
     return m_bot;
@@ -131,20 +110,6 @@ void ProfileEditorBackend::setLocked(bool locked)
     Q_EMIT lockedChanged();
 }
 
-QString ProfileEditorBackend::language() const
-{
-    return m_language;
-}
-
-void ProfileEditorBackend::setLanguage(const QString &language)
-{
-    if (language == m_language) {
-        return;
-    }
-    m_language = language;
-    Q_EMIT languageChanged();
-}
-
 QUrl ProfileEditorBackend::avatarUrl() const
 {
     return m_avatarUrl;
@@ -227,61 +192,48 @@ void ProfileEditorBackend::fetchAccountInfo()
         const auto json = QJsonDocument::fromJson(reply->readAll());
         Q_ASSERT(json.isObject());
         const auto obj = json.object();
-        setDisplayName(obj["display_name"].toString());
-        const auto source = obj["source"].toObject();
-        setSensitive(source["sensitive"].toBool());
-        setPrivacy(source["privacy"].toString());
-        setNote(source["note"].toString());
-        setLanguage(source["language"].toString());
-        setBot(obj["bot"].toBool());
-        setBackgroundUrl(obj["header_static"].toString());
-        setAvatarUrl(obj["avatar_static"].toString());
-        setLocked(obj["locked"].toBool());
-        setDiscoverable(obj["discoverable"].toBool());
+        setDisplayName(obj["display_name"_L1].toString());
+        const auto source = obj["source"_L1].toObject();
+        setNote(source["note"_L1].toString());
+        setBot(obj["bot"_L1].toBool());
+        setBackgroundUrl(QUrl(obj["header_static"_L1].toString()));
+        setAvatarUrl(QUrl(obj["avatar_static"_L1].toString()));
+        setLocked(obj["locked"_L1].toBool());
+        setDiscoverable(obj["discoverable"_L1].toBool());
     });
 }
 
 void ProfileEditorBackend::save()
 {
-    const QUrl url = m_account->apiUrl("/api/v1/accounts/update_credentials");
+    const QUrl url = m_account->apiUrl(QStringLiteral("/api/v1/accounts/update_credentials"));
 
     auto multiPart = new QHttpMultiPart(QHttpMultiPart::FormDataType);
 
     QHttpPart displayNamePart;
-    displayNamePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"display_name\""));
+    displayNamePart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"display_name\""));
     displayNamePart.setBody(displayName().toUtf8());
     multiPart->append(displayNamePart);
 
     QHttpPart notePart;
-    notePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"note\""));
+    notePart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"note\""));
     notePart.setBody(note().toUtf8());
     multiPart->append(notePart);
 
     QHttpPart lockedPart;
-    lockedPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"locked\""));
+    lockedPart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"locked\""));
     lockedPart.setBody(locked() ? "1" : "0");
     multiPart->append(lockedPart);
 
     QHttpPart botPart;
-    botPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"bot\""));
+    botPart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"bot\""));
     botPart.setBody(bot() ? "1" : "0");
     multiPart->append(botPart);
 
     QHttpPart discoverablePart;
-    discoverablePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"discoverable\""));
+    discoverablePart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"discoverable\""));
     discoverablePart.setBody(discoverable() ? "1" : "0");
     multiPart->append(discoverablePart);
 
-    QHttpPart privacyPart;
-    privacyPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"source[privacy]\""));
-    privacyPart.setBody(privacy().toUtf8());
-    multiPart->append(privacyPart);
-
-    QHttpPart sensitivityPart;
-    sensitivityPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"source[sensitive]\""));
-    sensitivityPart.setBody(sensitive() ? "1" : "0");
-    multiPart->append(sensitivityPart);
-
     QMimeDatabase mimeDatabase;
 
     if (backgroundUrl().isLocalFile()) {
@@ -290,14 +242,14 @@ void ProfileEditorBackend::save()
         if (file->open(QIODevice::ReadOnly)) {
             QHttpPart headerPart;
             headerPart.setHeader(QNetworkRequest::ContentTypeHeader, mime.name());
-            headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"header\""));
+            headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"header\""));
             headerPart.setBodyDevice(file);
             file->setParent(multiPart);
             multiPart->append(headerPart);
         }
     } else if (backgroundUrl().isEmpty()) {
         QHttpPart headerPart;
-        headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"header\""));
+        headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"header\""));
         multiPart->append(headerPart);
     }
 
@@ -307,14 +259,14 @@ void ProfileEditorBackend::save()
         if (file->open(QIODevice::ReadOnly)) {
             QHttpPart headerPart;
             headerPart.setHeader(QNetworkRequest::ContentTypeHeader, mime.name());
-            headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"avatar\""));
+            headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"avatar\""));
             headerPart.setBodyDevice(file);
             file->setParent(multiPart);
             multiPart->append(headerPart);
         }
     } else if (avatarUrl().isEmpty()) {
         QHttpPart headerPart;
-        headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"avatar\""));
+        headerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("form-data; name=\"avatar\""));
         multiPart->append(headerPart);
     }
 
