@@ -14,6 +14,8 @@
 #include <QNetworkReply>
 #include <qstringliteral.h>
 
+using namespace Qt::Literals::StringLiterals;
+
 SocialGraphModel::SocialGraphModel(QObject *parent)
     : QAbstractListModel(parent)
 {
@@ -26,36 +28,44 @@ QString SocialGraphModel::name() const
 
 QString SocialGraphModel::displayName() const
 {
-    if (m_followListName == "request") {
+    if (m_followListName == QStringLiteral("request")) {
         return i18nc("@title", "Follow Requests");
-    } else if (m_followListName == "followers") {
+    } else if (m_followListName == QStringLiteral("followers")) {
         return i18nc("@title", "Followers");
-    } else if (m_followListName == "following") {
+    } else if (m_followListName == QStringLiteral("following")) {
         return i18nc("@title", "Following");
-    } else if (m_followListName == "mutes") {
-        return i18nc("@title", "Muted Accounts");
-    } else if (m_followListName == "blocks") {
-        return i18nc("@title", "Blocked Accounts");
-    } else if (m_followListName == "featured") {
-        return i18nc("@title", "Featured Accounts");
+    } else if (m_followListName == QStringLiteral("mutes")) {
+        return i18nc("@title", "Muted Users");
+    } else if (m_followListName == QStringLiteral("blocks")) {
+        return i18nc("@title", "Blocked Users");
+    } else if (m_followListName == QStringLiteral("featured")) {
+        return i18nc("@title", "Featured Users");
+    } else if (m_followListName == QStringLiteral("favourited_by")) {
+        return i18ncp("@title", "%1 favourite", "%1 favourites", m_count);
+    } else if (m_followListName == QStringLiteral("reblogged_by")) {
+        return i18ncp("@title", "%1 boost", "%1 boosts", m_count);
     }
     return {};
 }
 
 QString SocialGraphModel::placeholderText() const
 {
-    if (m_followListName == "request") {
+    if (m_followListName == QStringLiteral("request")) {
         return i18n("No follow requests");
-    } else if (m_followListName == "followers") {
+    } else if (m_followListName == QStringLiteral("followers")) {
         return i18n("No followers");
-    } else if (m_followListName == "following") {
-        return i18n("No followed accounts");
-    } else if (m_followListName == "mutes") {
-        return i18n("No muted accounts");
-    } else if (m_followListName == "blocks") {
-        return i18n("No blocked accounts");
-    } else if (m_followListName == "featured") {
-        return i18n("No featured accounts");
+    } else if (m_followListName == QStringLiteral("following")) {
+        return i18n("No followed users");
+    } else if (m_followListName == QStringLiteral("mutes")) {
+        return i18n("No muted users");
+    } else if (m_followListName == QStringLiteral("blocks")) {
+        return i18n("No blocked users");
+    } else if (m_followListName == QStringLiteral("featured")) {
+        return i18n("No featured users");
+    } else if (m_followListName == QStringLiteral("favourited_by")) {
+        return i18n("No users favourited this post");
+    } else if (m_followListName == QStringLiteral("reblogged_by")) {
+        return i18n("No users boosted this post");
     }
     return {};
 }
@@ -83,6 +93,28 @@ void SocialGraphModel::setAccountId(const QString &accountId)
     fillTimeline();
 }
 
+QString SocialGraphModel::statusId() const
+{
+    return m_statusId;
+}
+
+void SocialGraphModel::setStatusId(const QString &statusId)
+{
+    m_statusId = statusId;
+    Q_EMIT statusIdChanged();
+    fillTimeline();
+}
+
+int SocialGraphModel::count() const
+{
+    return m_count;
+}
+
+void SocialGraphModel::setCount(int count)
+{
+    m_count = count;
+}
+
 QVariant SocialGraphModel::data(const QModelIndex &index, int role) const
 {
     Q_ASSERT(checkIndex(index, QAbstractItemModel::CheckIndexOption::IndexIsValid));
@@ -124,17 +156,17 @@ void SocialGraphModel::setLoading(bool loading)
 
 bool SocialGraphModel::isFollowRequest() const
 {
-    return m_followListName == "request";
+    return m_followListName == QStringLiteral("request");
 }
 
 bool SocialGraphModel::isFollowing() const
 {
-    return m_followListName == "following";
+    return m_followListName == QStringLiteral("following");
 }
 
 bool SocialGraphModel::isFollower() const
 {
-    return m_followListName == "followers";
+    return m_followListName == QStringLiteral("followers");
 }
 
 void SocialGraphModel::actionAllow(const QModelIndex &index)
@@ -147,7 +179,7 @@ void SocialGraphModel::actionAllow(const QModelIndex &index)
     auto requestIdentity = m_accounts[index.row()].get();
     const auto requestIdentityId = requestIdentity->id();
 
-    account->post(account->apiUrl(QString("/api/v1/follow_requests/%1/authorize").arg(requestIdentityId)),
+    account->post(account->apiUrl(QStringLiteral("/api/v1/follow_requests/%1/authorize").arg(requestIdentityId)),
                   QJsonDocument{},
                   true,
                   this,
@@ -160,6 +192,8 @@ void SocialGraphModel::actionAllow(const QModelIndex &index)
                       m_accounts.removeAt(index.row());
                       endRemoveRows();
                   });
+
+    account->checkForFollowRequests();
 }
 
 void SocialGraphModel::actionDeny(const QModelIndex &index)
@@ -172,7 +206,7 @@ void SocialGraphModel::actionDeny(const QModelIndex &index)
     auto requestIdentity = m_accounts[index.row()].get();
     const auto requestIdentityId = requestIdentity->id();
 
-    account->post(account->apiUrl(QString("/api/v1/follow_requests/%1/reject").arg(requestIdentityId)),
+    account->post(account->apiUrl(QStringLiteral("/api/v1/follow_requests/%1/reject").arg(requestIdentityId)),
                   QJsonDocument{},
                   true,
                   this,
@@ -185,6 +219,8 @@ void SocialGraphModel::actionDeny(const QModelIndex &index)
                       m_accounts.removeAt(index.row());
                       endRemoveRows();
                   });
+
+    account->checkForFollowRequests();
 }
 
 bool SocialGraphModel::canFetchMore(const QModelIndex &parent) const
@@ -208,7 +244,13 @@ void SocialGraphModel::fillTimeline()
         return;
     }
 
-    if ((m_followListName == "followers" || m_followListName == "following") && (m_accountId.isEmpty() || m_accountId.isNull())) {
+    if ((m_followListName == QStringLiteral("followers") || m_followListName == QStringLiteral("following"))
+        && (m_accountId.isEmpty() || m_accountId.isNull())) {
+        return;
+    }
+
+    if ((m_followListName == QStringLiteral("favourited_by") || m_followListName == QStringLiteral("reblogged_by"))
+        && (m_statusId.isEmpty() || m_statusId.isNull())) {
         return;
     }
 
@@ -218,18 +260,22 @@ void SocialGraphModel::fillTimeline()
     setLoading(true);
 
     QString uri;
-    if (m_followListName == "request") {
-        uri = "/api/v1/follow_requests";
-    } else if (m_followListName == "followers") {
+    if (m_followListName == QStringLiteral("request")) {
+        uri = QStringLiteral("/api/v1/follow_requests");
+    } else if (m_followListName == QStringLiteral("followers")) {
         uri = QStringLiteral("/api/v1/accounts/%1/followers").arg(m_accountId);
-    } else if (m_followListName == "following") {
+    } else if (m_followListName == QStringLiteral("following")) {
         uri = QStringLiteral("/api/v1/accounts/%1/following").arg(m_accountId);
-    } else if (m_followListName == "mutes") {
+    } else if (m_followListName == QStringLiteral("mutes")) {
         uri = QStringLiteral("/api/v1/mutes");
-    } else if (m_followListName == "blocks") {
+    } else if (m_followListName == QStringLiteral("blocks")) {
         uri = QStringLiteral("/api/v1/blocks");
-    } else if (m_followListName == "featured") {
+    } else if (m_followListName == QStringLiteral("featured")) {
         uri = QStringLiteral("/api/v1/endorsements");
+    } else if (m_followListName == QStringLiteral("favourited_by")) {
+        uri = QStringLiteral("/api/v1/statuses/%1/favourited_by").arg(m_statusId);
+    } else if (m_followListName == QStringLiteral("reblogged_by")) {
+        uri = QStringLiteral("/api/v1/statuses/%1/reblogged_by").arg(m_statusId);
     }
 
     QUrl url;
@@ -244,9 +290,9 @@ void SocialGraphModel::fillTimeline()
         const auto accounts = followRequestResult.array();
 
         if (!accounts.isEmpty()) {
-            static QRegularExpression re("<(.*)>; rel=\"next\"");
+            static QRegularExpression re(QStringLiteral("<(.*)>; rel=\"next\""));
             const auto next = reply->rawHeader(QByteArrayLiteral("Link"));
-            const auto match = re.match(next);
+            const auto match = re.match(QString::fromUtf8(next));
             if (re.isValid()) {
                 m_next = QUrl::fromUserInput(match.captured(1));
             }
@@ -259,7 +305,7 @@ void SocialGraphModel::fillTimeline()
                 std::back_inserter(fetchedAccounts),
                 [account](const QJsonValue &value) -> auto{
                     const auto identityJson = value.toObject();
-                    return account->identityLookup(identityJson["id"].toString(), identityJson);
+                    return account->identityLookup(identityJson["id"_L1].toString(), identityJson);
                 });
 
             beginInsertRows({}, m_accounts.size(), m_accounts.size() + fetchedAccounts.size() - 1);
