@@ -9,7 +9,7 @@ class MockAccount : public AbstractAccount
     Q_OBJECT
 
 public:
-    MockAccount(QObject *parent = nullptr);
+    explicit MockAccount(QObject *parent = nullptr);
     void get(const QUrl &url,
              bool authenticated,
              QObject *parent,
@@ -24,11 +24,17 @@ public:
               std::function<void(QNetworkReply *)> errorCallback,
               QHash<QByteArray, QByteArray> headers = {}) override;
 
-    void post(const QUrl &url, const QUrlQuery &formdata, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
+    void post(const QUrl &url,
+              const QUrlQuery &formdata,
+              bool authenticated,
+              QObject *parent,
+              std::function<void(QNetworkReply *)> callback,
+              std::function<void(QNetworkReply *)> errorCallback = nullptr) override;
 
     QNetworkReply *post(const QUrl &url, QHttpMultiPart *message, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
 
     void put(const QUrl &url, const QJsonDocument &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
+    void put(const QUrl &url, const QUrlQuery &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) override;
 
     QNetworkReply *upload(const QUrl &filename, std::function<void(QNetworkReply *)> callback) override;
 
@@ -38,7 +44,7 @@ public:
 
     void writeToSettings() override;
 
-    void buildFromSettings(const AccountConfig &settings) override;
+    void buildFromSettings() override;
 
     void validateToken() override;
 
@@ -53,7 +59,20 @@ public:
     void setFakeIdentity(const QJsonObject &object);
     void clearFakeIdentity();
 
+    void updatePushNotifications() override {};
+
+    Q_INVOKABLE void mentionNotification();
+    Q_INVOKABLE void favoriteNotification();
+    Q_INVOKABLE void boostNotification();
+    Q_INVOKABLE void followNotification();
+    Q_INVOKABLE void followRequestNotification();
+    Q_INVOKABLE void statusNotification();
+    Q_INVOKABLE void updateNotification();
+    Q_INVOKABLE void pollNotification();
+
 private:
+    void readNotificationFromFile(QLatin1String filename);
+
     QHash<QUrl, QNetworkReply *> m_postReplies;
     QHash<QUrl, QNetworkReply *> m_getReplies;
 };
