@@ -37,6 +37,10 @@ QVariant AttachmentEditorModel::data(const QModelIndex &index, int role) const
         return attachment->m_preview_url;
     case DescriptionRole:
         return attachment->description();
+    case FocalXRole:
+        return attachment->focusX();
+    case FocalYRole:
+        return attachment->focusY();
     }
 
     return {};
@@ -47,15 +51,17 @@ QHash<int, QByteArray> AttachmentEditorModel::roleNames() const
     return {
         {PreviewRole, QByteArrayLiteral("preview")},
         {DescriptionRole, QByteArrayLiteral("description")},
+        {FocalXRole, QByteArrayLiteral("focalX")},
+        {FocalYRole, QByteArrayLiteral("focalY")},
     };
 }
 
-QNetworkReply *AttachmentEditorModel::append(const QUrl &filename)
+QNetworkReply *AttachmentEditorModel::append(const QString &filename)
 {
     if (rowCount({}) >= 4) {
         return nullptr;
     }
-    return m_account->upload(filename, [=](QNetworkReply *reply) {
+    return m_account->upload(QUrl(filename), [=](QNetworkReply *reply) {
         const auto doc = QJsonDocument::fromJson(reply->readAll());
 
         if (!doc.isObject()) {
@@ -93,14 +99,30 @@ void AttachmentEditorModel::setDescription(int row, const QString &description)
 
     const auto attachementUrl = m_account->apiUrl(QStringLiteral("/api/v1/media/%1").arg(id));
     const QJsonObject obj{
-        {"description", description},
+        {QStringLiteral("description"), description},
     };
     const QJsonDocument doc(obj);
     m_account->put(attachementUrl, doc, true, this, nullptr);
     Q_EMIT dataChanged(index(row, 0), index(row, 0), {DescriptionRole});
 }
 
-const QVector<Attachment *> &AttachmentEditorModel::attachments() const
+void AttachmentEditorModel::setFocusPoint(int row, double x, double y)
+{
+    auto &attachment = m_attachments[row];
+    const auto id = attachment->id();
+    attachment->setFocusX(x);
+    attachment->setFocusY(y);
+
+    const auto attachementUrl = m_account->apiUrl(QStringLiteral("/api/v1/media/%1").arg(id));
+    const QJsonObject obj{
+        {QStringLiteral("focus"), QStringLiteral("%1,%2").arg(x).arg(y)},
+    };
+    const QJsonDocument doc(obj);
+    m_account->put(attachementUrl, doc, true, this, nullptr);
+    Q_EMIT dataChanged(index(row, 0), index(row, 0), {FocalXRole, FocalYRole});
+}
+
+const QList<Attachment *> &AttachmentEditorModel::attachments() const
 {
     return m_attachments;
 }
