@@ -1,5 +1,5 @@
 // SPDX-FileCopyrightText: 2022 Gary Wang <wzc782970009@gmail.com>
-// SPDX-License-Identifier: GPL-2.0-or-later OR GPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+// SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 #include "networkcontroller.h"
 
@@ -10,6 +10,9 @@
 
 #include <QNetworkProxy>
 #include <QUrlQuery>
+#include <utility>
+
+using namespace Qt::Literals::StringLiterals;
 
 NetworkController::NetworkController(QObject *parent)
     : QObject(parent)
@@ -19,7 +22,13 @@ NetworkController::NetworkController(QObject *parent)
     connect(&AccountManager::instance(), &AccountManager::accountsReady, this, [=] {
         m_accountsReady = true;
         openLink();
+
+        if (!m_storedComposedText.isEmpty()) {
+            Q_EMIT openComposer(m_storedComposedText);
+            m_storedComposedText.clear();
+        }
     });
+    m_accountsReady = AccountManager::instance().isReady();
 }
 
 NetworkController &NetworkController::instance()
@@ -60,19 +69,31 @@ void NetworkController::setApplicationProxy()
     AccountManager::instance().reloadAccounts();
 }
 
-void NetworkController::openWebApLink(QString url)
+void NetworkController::openWebApLink(QString input)
 {
-    if (url.startsWith("web+ap")) {
-        url = url.replace(QRegularExpression("(web\\+ap)+:\\/\\/"), "https://");
+    QUrl url(input);
+    // only web+ap (declared in app manifest) and https (explicitly not declared, can be used from command line)
+    if (url.scheme() != QStringLiteral("web+ap") && url.scheme() != QStringLiteral("https")) {
+        // FIXME maybe warn about unsupported links?
+        return;
     }
 
-    m_requestedLink = url;
+    m_requestedLink = std::move(url);
 
     if (m_accountsReady) {
         openLink();
     }
 }
 
+void NetworkController::setAuthCode(QUrl authCode)
+{
+    QUrlQuery query(authCode);
+
+    if (query.hasQueryItem(QStringLiteral("code"))) {
+        Q_EMIT receivedAuthCode(query.queryItemValue(QStringLiteral("code")));
+    }
+}
+
 void NetworkController::openLink()
 {
     if (m_requestedLink.isEmpty())
@@ -80,13 +101,17 @@ void NetworkController::openLink()
 
     auto account = AccountManager::instance().selectedAccount();
 
-    auto url = account->apiUrl("/api/v2/search");
-    url.setQuery({
-        {"q", m_requestedLink},
-        {"resolve", "true"},
-        {"limit", "1"},
-    });
-    account->get(url, true, &AccountManager::instance(), [=](QNetworkReply *reply) {
+    if (m_requestedLink.scheme() == QStringLiteral("web+ap")) {
+        if (m_requestedLink.userName() == QStringLiteral("tag")) {
+            // TODO implement in a future MR
+            m_requestedLink.clear();
+            return;
+        }
+        m_requestedLink.setScheme(QStringLiteral("https"));
+        m_requestedLink.setUserInfo(QString());
+    }
+
+    requestRemoteObject(account, m_requestedLink.toString(), [=](QNetworkReply *reply) {
         const auto searchResult = QJsonDocument::fromJson(reply->readAll()).object();
         const auto statuses = searchResult[QStringLiteral("statuses")].toArray();
         const auto accounts = searchResult[QStringLiteral("accounts")].toArray();
@@ -96,7 +121,7 @@ void NetworkController::openLink()
         } else {
             const auto status = statuses[0].toObject();
 
-            Q_EMIT NetworkController::instance().openPost(status["id"].toString());
+            Q_EMIT NetworkController::instance().openPost(status["id"_L1].toString());
         }
 
         if (accounts.isEmpty()) {
@@ -104,9 +129,29 @@ void NetworkController::openLink()
         } else {
             const auto account = accounts[0].toObject();
 
-            Q_EMIT NetworkController::instance().openAccount(account["id"].toString());
+            Q_EMIT NetworkController::instance().openAccount(account["id"_L1].toString());
         }
 
         m_requestedLink.clear();
     });
 }
+
+void NetworkController::startComposing(const QString &text)
+{
+    if (m_accountsReady) {
+        Q_EMIT openComposer(text);
+    } else {
+        m_storedComposedText = text;
+    }
+}
+
+void NetworkController::requestRemoteObject(AbstractAccount *account, const QString &remoteUrl, std::function<void(QNetworkReply *)> callback)
+{
+    auto url = account->apiUrl(QStringLiteral("/api/v2/search"));
+    url.setQuery({
+        {QStringLiteral("q"), remoteUrl},
+        {QStringLiteral("resolve"), QStringLiteral("true")},
+        {QStringLiteral("limit"), QStringLiteral("1")},
+    });
+    account->get(url, true, &AccountManager::instance(), std::move(callback));
+}
