@@ -4,12 +4,14 @@
 #pragma once
 
 #include <QObject>
+#include <QtQml/qqmlregistration.h>
 
 class QNetworkReply;
 
 class NetworkRequestProgress : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
 
     Q_PROPERTY(QNetworkReply *reply READ reply WRITE setReply NOTIFY replyChanged)
 
