@@ -12,6 +12,8 @@
 #include <QtMath>
 #include <qvariant.h>
 
+using namespace Qt::Literals::StringLiterals;
+
 AbstractTimelineModel::AbstractTimelineModel(QObject *parent)
     : QAbstractListModel(parent)
 {
@@ -42,6 +44,8 @@ QHash<int, QByteArray> AbstractTimelineModel::roleNames() const
         {AuthorIdentityRole, QByteArrayLiteral("authorIdentity")},
         {PublishedAtRole, QByteArrayLiteral("publishedAt")},
         {VisibilityRole, QByteArrayLiteral("visibility")},
+        {WasEditedRole, QByteArrayLiteral("wasEdited")},
+        {EditedAtRole, QByteArrayLiteral("editedAt")},
         {SelectedRole, QByteArrayLiteral("selected")},
         {FiltersRole, QByteArrayLiteral("filters")},
         {RelativeTimeRole, QByteArrayLiteral("relativeTime")},
@@ -78,8 +82,10 @@ QHash<int, QByteArray> AbstractTimelineModel::roleNames() const
         // Notification
         {NotificationActorIdentityRole, "notificationActorIdentity"},
         {TypeRole, "type"},
-
         {PostRole, "post"},
+        {IsGroupRole, "isGroup"},
+        {NumInGroupRole, "numInGroup"},
+        {IsInGroupRole, "isInGroup"},
     };
 }
 
@@ -133,6 +139,10 @@ QVariant AbstractTimelineModel::postData(Post *post, int role) const
         return post->spoilerText();
     case VisibilityRole:
         return post->visibility();
+    case WasEditedRole:
+        return post->wasEdited();
+    case EditedAtRole:
+        return post->editedAt();
     case FiltersRole:
         return post->filters();
     case AttachmentsRole:
@@ -162,7 +172,7 @@ QVariant AbstractTimelineModel::postData(Post *post, int role) const
         return {};
     case TypeRole:
     case NotificationActorIdentityRole:
-        return false;
+        return {};
     case PostRole:
         return QVariant::fromValue<Post *>(post);
     }
@@ -198,33 +208,36 @@ void AbstractTimelineModel::actionRepeat(const QModelIndex &index, Post *post)
 
 void AbstractTimelineModel::actionRedraft(const QModelIndex &index, Post *post, bool isEdit)
 {
-    m_account->get(m_account->apiUrl(QString("/api/v1/statuses/%1/source").arg(post->postId())), true, this, [this, post, index, isEdit](QNetworkReply *reply) {
-        const auto postSource = QJsonDocument::fromJson(reply->readAll()).object();
+    m_account->get(m_account->apiUrl(QStringLiteral("/api/v1/statuses/%1/source").arg(post->postId())),
+                   true,
+                   this,
+                   [this, post, index, isEdit](QNetworkReply *reply) {
+                       const auto postSource = QJsonDocument::fromJson(reply->readAll()).object();
 
-        auto backend = new PostEditorBackend();
-        backend->setId(post->postId());
-        backend->setStatus(postSource["text"].toString());
-        backend->setSpoilerText(postSource["spoiler_text"].toString());
-        backend->setInReplyTo(post->inReplyTo());
-        backend->setVisibility(post->visibility());
-        backend->setLanguage(post->language());
-        backend->setMentions(post->mentions()); // TODO: needed?
-        backend->setSensitive(post->sensitive());
+                       auto backend = new PostEditorBackend();
+                       backend->setId(post->postId());
+                       backend->setStatus(postSource["text"_L1].toString());
+                       backend->setSpoilerText(postSource["spoiler_text"_L1].toString());
+                       backend->setInReplyTo(post->inReplyTo());
+                       backend->setVisibility(post->visibility());
+                       backend->setLanguage(post->language());
+                       backend->setMentions(post->mentions()); // TODO: needed?
+                       backend->setSensitive(post->sensitive());
 
-        Q_EMIT postSourceReady(backend, isEdit);
+                       Q_EMIT postSourceReady(backend, isEdit);
 
-        auto attachmentBackend = backend->attachmentEditorModel();
-        for (const auto &attachment : post->attachments()) {
-            attachmentBackend->appendExisting(attachment);
-        }
+                       auto attachmentBackend = backend->attachmentEditorModel();
+                       for (const auto &attachment : post->attachments()) {
+                           attachmentBackend->appendExisting(attachment);
+                       }
 
-        if (isEdit) {
-            connect(backend, &PostEditorBackend::editComplete, this, [this, post, index](QJsonObject object) {
-                post->fromJson(object);
-                Q_EMIT dataChanged(index, index);
-            });
-        }
-    });
+                       if (isEdit) {
+                           connect(backend, &PostEditorBackend::editComplete, this, [this, post, index](QJsonObject object) {
+                               post->fromJson(object);
+                               Q_EMIT dataChanged(index, index);
+                           });
+                       }
+                   });
 }
 
 void AbstractTimelineModel::actionBookmark(const QModelIndex &index, Post *post)
@@ -256,8 +269,5 @@ void AbstractTimelineModel::actionPin(const QModelIndex &index, Post *post)
 void AbstractTimelineModel::actionDelete(const QModelIndex &index, Post *post)
 {
     Q_UNUSED(index);
-    m_account->deleteResource(m_account->apiUrl(QString("/api/v1/statuses/%1").arg(post->postId())), true, this, [](QNetworkReply *reply) {
-        const auto postSource = QJsonDocument::fromJson(reply->readAll()).object();
-        qDebug() << "DELETED: " << postSource;
-    });
+    m_account->deleteResource(m_account->apiUrl(QStringLiteral("/api/v1/statuses/%1").arg(post->postId())), true, this, {});
 }
\ No newline at end of file
