@@ -3,6 +3,8 @@
 
 #pragma once
 
+#include <QtQml/qqmlregistration.h>
+
 #include "timelinemodel.h"
 
 class AbstractAccount;
@@ -12,7 +14,10 @@ class AbstractAccount;
 class MainTimelineModel : public TimelineModel
 {
     Q_OBJECT
+    QML_ELEMENT
+
     Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
+    Q_PROPERTY(bool atEnd READ atEnd NOTIFY atEndChanged)
 
 public:
     explicit MainTimelineModel(QObject *parent = nullptr);
@@ -28,6 +33,11 @@ public:
     QString displayName() const override;
     void handleEvent(AbstractAccount::StreamingEventType eventType, const QByteArray &payload) override;
 
+    bool atEnd() const;
+
+Q_SIGNALS:
+    void atEndChanged();
+
 private:
     QString m_timelineName;
     QUrl m_next;
