@@ -5,20 +5,21 @@
 
 #include <QDateTime>
 #include <QJsonObject>
+#include <QtQml/qqmlregistration.h>
 
 class Poll
 {
     Q_GADGET
 
-    Q_PROPERTY(QString id READ id CONSTANT);
-    Q_PROPERTY(QDateTime expiresAt READ expiresAt CONSTANT);
-    Q_PROPERTY(bool expired READ expired CONSTANT);
-    Q_PROPERTY(bool multiple READ multiple CONSTANT);
-    Q_PROPERTY(int votesCount READ votesCount CONSTANT);
-    Q_PROPERTY(int votersCount READ votersCount CONSTANT);
-    Q_PROPERTY(bool voted READ voted CONSTANT);
-    Q_PROPERTY(QList<int> ownVotes READ ownVotes CONSTANT);
-    Q_PROPERTY(QList<QVariantMap> options READ options CONSTANT);
+    Q_PROPERTY(QString id READ id CONSTANT)
+    Q_PROPERTY(QDateTime expiresAt READ expiresAt CONSTANT)
+    Q_PROPERTY(bool expired READ expired CONSTANT)
+    Q_PROPERTY(bool multiple READ multiple CONSTANT)
+    Q_PROPERTY(int votesCount READ votesCount CONSTANT)
+    Q_PROPERTY(int votersCount READ votersCount CONSTANT)
+    Q_PROPERTY(bool voted READ voted CONSTANT)
+    Q_PROPERTY(QList<int> ownVotes READ ownVotes CONSTANT)
+    Q_PROPERTY(QList<QVariantMap> options READ options CONSTANT)
 
 public:
     Poll();
@@ -45,5 +46,3 @@ private:
     QList<int> m_ownVotes;
     QList<QVariantMap> m_options;
 };
-
-Q_DECLARE_METATYPE(Poll *)
\ No newline at end of file
