@@ -4,6 +4,8 @@
 
 #pragma once
 
+#include <QtQml/qqmlregistration.h>
+
 #include "timelinemodel.h"
 
 /// Model for displaying and organizing post threads
@@ -11,6 +13,7 @@
 class ThreadModel : public TimelineModel
 {
     Q_OBJECT
+    QML_ELEMENT
 
     Q_PROPERTY(QString postId READ postId WRITE setPostId NOTIFY postIdChanged)
 
@@ -31,6 +34,10 @@ public:
     void fillTimeline(const QString &fromId = QString()) override;
     bool canFetchMore(const QModelIndex &parent) const override;
 
+    /// Returns the index of the root post in the model
+    /// Can be used to find where exactly to position the view
+    Q_INVOKABLE int getRootIndex() const;
+
 Q_SIGNALS:
     void postIdChanged();
 
