@@ -1,13 +1,18 @@
 // SPDX-FileCopyrightText: 2022 Carl Schwan <carl@carlschwan.eu>
 // SPDX-License-Identifier: LGPL-2.0-or-later
 
+#pragma once
+
 #include <QObject>
+#include <QtQml/qqmlregistration.h>
 
 class AbstractAccount;
 
 class FileHelper : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
 
 public:
     explicit FileHelper(QObject *parent = nullptr);
